@@ -4,8 +4,8 @@
 // Author:      Mattia Barbon
 // Modified by:
 // Created:     29/10/2000
-// RCS-ID:      $Id: e_cback.cpp 2057 2007-06-18 23:03:00Z mbarbon $
-// Copyright:   (c) 2000-2002, 2004-2007 Mattia Barbon
+// RCS-ID:      $Id: e_cback.cpp 2454 2008-08-31 11:12:47Z mbarbon $
+// Copyright:   (c) 2000-2002, 2004-2008 Mattia Barbon
 // Licence:     This program is free software; you can redistribute it and/or
 //              modify it under the same terms as Perl itself
 /////////////////////////////////////////////////////////////////////////////
@@ -17,6 +17,7 @@ wxPliEventCallback::wxPliEventCallback( SV* method, SV* self )
     SvREFCNT_inc( m_method );
     m_self = self;
     SvREFCNT_inc( m_self );
+    m_is_method = !SvROK( m_method ) || !SvRV( m_method );
 }
 
 wxPliEventCallback::~wxPliEventCallback() 
@@ -108,7 +109,14 @@ void wxPliEventCallback::Handler( wxEvent& event )
         XPUSHs( e );
         PUTBACK;
 
-        call_sv( This->m_method, G_EVAL|G_VOID|G_DISCARD );
+        if( This->m_is_method )
+        {
+            call_method( SvPV_nolen( This->m_method ), G_EVAL|G_VOID|G_DISCARD );
+        }
+        else
+        {
+            call_sv( This->m_method, G_EVAL|G_VOID|G_DISCARD );
+        }
 
         SPAGAIN;
     }
